from pyleap.shape.circle import Circle
from pyleap.shape.line import Line
from pyleap.shape.point import Point
from pyleap.shape.rectangle import Rectangle
from pyleap.shape.shape import Shape
from pyleap.shape.sprite import Sprite
from pyleap.shape.text import Text
from pyleap.shape.polygon import Polygon, Triangle
from pyleap.shape.ellipse import Ellipse